# Generated by Django 3.2.5 on 2021-07-13 02:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0047_auto_20210531_1025'),
    ]

    operations = [
        migrations.CreateModel(
            name='MailLabel',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('label_id', models.IntegerField(default=None)),
                ('label_name', models.CharField(
                    default=None, max_length=255, null=True)),
                ('label_color', models.CharField(
                    default=None, max_length=7, null=True)),
                ('unread_count', models.IntegerField(default=None, null=True)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
            ],
        ),
        migrations.CreateModel(
            name='MailRecipient',
            fields=[
                ('recipient_id', models.BigIntegerField(
                    primary_key=True, serialize=False, unique=True)),
                ('recipient_type', models.CharField(choices=[('alliance', 'alliance'), ('character', 'character'), (
                    'corporation', 'corporation'), ('mailing_list', 'mailing_list')], max_length=15)),
            ],
        ),
        migrations.CreateModel(
            name='MailMessage',
            fields=[
                ('id_key', models.BigIntegerField(
                    primary_key=True, serialize=False, unique=True)),
                ('mail_id', models.IntegerField(default=None, null=True)),
                ('from_id', models.IntegerField(default=None, null=True)),
                ('is_read', models.BooleanField(default=False)),
                ('timestamp', models.DateTimeField(default=None, null=True)),
                ('subject', models.CharField(default=None, max_length=255, null=True)),
                ('body', models.CharField(default=None, max_length=12000, null=True)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
                ('labels', models.ManyToManyField(to='corptools.MailLabel')),
                ('recipients', models.ManyToManyField(
                    to='corptools.MailRecipient')),
            ],
        ),
    ]



from celery import app
from django.conf.urls import url, include

from . import views
from corptools.audit_views.character import assets, wallet, pub_data, contacts, skills, clones, assets_lists, roles, market, status, notifications
from corptools.audit_views.corporation import corp_list
from . import app_settings
app_name = 'corptools'

_character_ulrs = [
    url(r'^pub/$', pub_data, name='public_data'),
    url(r'^overview/$', status, name='overview'),
]

if app_settings.CT_CHAR_ASSETS_MODULE:
    _character_ulrs += [
        url(r'^assets/$', assets, name='assets'),
        url(r'^assets_lists/(?P<location_id>(\d)*)/$',
            assets_lists, name='assets_lists'),
    ]

if app_settings.CT_CHAR_WALLET_MODULE:
    _character_ulrs += [
        url(r'^market/$', market, name='market'),
        url(r'^wallet/$', wallet, name='wallet'),
    ]

if app_settings.CT_CHAR_ROLES_MODULE:
    _character_ulrs += [
        url(r'^roles/$', roles, name='roles'),
    ]

if app_settings.CT_CHAR_NOTIFICATIONS_MODULE:
    _character_ulrs += [
        url(r'^notes/$', notifications, name='notifications'),
    ]

if app_settings.CT_CHAR_CLONES_MODULE:
    _character_ulrs += [
        url(r'^clones/$', clones, name='clones'),
    ]

if app_settings.CT_CHAR_SKILLS_MODULE:
    _character_ulrs += [
        url(r'^skills/$', skills, name='skills'),
    ]

if app_settings.CT_CHAR_CONTACTS_MODULE:
    _character_ulrs += [
        url(r'^contacts/$', contacts, name='contacts'),
    ]

urlpatterns = [
    url(r'^$', views.corptools_menu, name='view'),
    url(r'^check_account/(?P<character_id>(\d)*)/$',
        views.update_account, name='update_account'),
    url(r'^admin/$', views.admin, name='admin'),
    url(r'^admin_add_pyfa_xml/$', views.admin_add_pyfa_xml,
        name='admin_add_pyfa_xml'),
    url(r'^admin_create_tasks/$', views.admin_create_tasks,
        name='admin_create_tasks'),
    url(r'^run_tasks/$', views.admin_run_tasks, name='run_tasks'),
    url(r'^char/add/$', views.add_char, name='add_char'),
    url(r'^char/(?P<character_id>(\d)*)/', include(_character_ulrs)),
    url(r'^corp/', include([
        url(r'^menu/', corp_list, name='corp_menu'),
        url(r'^add/$', views.add_corp, name='add_corp'),
    ])),
]



# Generated by Django 3.1.1 on 2021-04-27 07:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0012_index_additions'),
        ('corptools', '0045_bigkey_assets_fix'),
    ]

    operations = [
        migrations.AddField(
            model_name='characteraudit',
            name='cache_expire_orders',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_orders',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.CreateModel(
            name='Rolefilter',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('has_director', models.BooleanField(default=False)),
                ('has_accountant', models.BooleanField(default=False)),
                ('has_station_manager', models.BooleanField(default=False)),
                ('has_personnel_manager', models.BooleanField(default=False)),
                ('alliance_filter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                      related_name='audit_role_filter', to='eveonline.eveallianceinfo')),
                ('corp_filter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                  related_name='audit_role_filter', to='eveonline.evecorporationinfo')),
            ],
            options={
                'verbose_name': 'Smart Filter: Corporate Role checks',
                'verbose_name_plural': 'Smart Filter: Corporate Role checks',
            },
        ),
    ]



from allianceauth.services.hooks import MenuItemHook, UrlHook
from django.utils.translation import ugettext_lazy as _
from allianceauth import hooks
from . import urls
from . import models
from . import app_settings


class MemberAudit(MenuItemHook):
    def __init__(self):

        MenuItemHook.__init__(self,
                              _('Audit'),
                              'far fa-eye fa-fw',
                              'corptools:view',
                              navactive=['corptools:'])

    def render(self, request):
        if request.user.has_perm('corptools.view_characteraudit'):
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return MemberAudit()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'corptools', r'^audit/')


@hooks.register("secure_group_filters")
def filters():
    return [models.AssetsFilter, models.FullyLoadedFilter, models.Skillfilter, models.TimeInCorpFilter, models.Rolefilter, models.Titlefilter]


@hooks.register('discord_cogs_hook')
def register_cogs():
    return app_settings.CORPTOOLS_DISCORD_BOT_COGS



import os

from bravado.exception import HTTPError
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required, user_passes_test
from django.core.exceptions import PermissionDenied
from django.db import IntegrityError
from django.db.models import Count, F, Sum, Max, Q
from django.shortcuts import render, redirect, get_object_or_404
from django.utils.translation import ugettext_lazy as _
from esi.decorators import token_required
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from django.http import HttpResponse
import csv
import re
import copy

from ..models import *
from .. import providers


@login_required
def corp_list(request, character_id=None):

    corps = CorporationAudit.objects.visible_to(request.user)

    context = {
        "corporations": corps,
    }

    return render(request, 'corptools/corp_menu.html', context=context)



# Generated by Django 2.2.9 on 2020-05-11 08:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0008_auto_20200511_0801'),
    ]

    operations = [
        migrations.AddField(
            model_name='characterasset',
            name='type_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='corptools.EveItemType'),
        ),
        migrations.AddField(
            model_name='corpasset',
            name='type_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='corptools.EveItemType'),
        ),
    ]



# Generated by Django 3.1.6 on 2021-03-22 08:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0043_auto_20210125_0758'),
    ]

    operations = [
        migrations.AddField(
            model_name='characteraudit',
            name='cache_expire_roles',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_roles',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.CreateModel(
            name='CharacterTitle',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('title_id', models.IntegerField()),
                ('title', models.CharField(max_length=500)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
            ],
        ),
        migrations.CreateModel(
            name='CharacterRoles',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('director', models.BooleanField(default=False)),
                ('accountant', models.BooleanField(default=False)),
                ('station_manager', models.BooleanField(default=False)),
                ('personnel_manager', models.BooleanField(default=False)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
            ],
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-16 10:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0011_characterwalletjournalentry_corporationwalletdivision_corporationwalletjournalentry'),
    ]

    operations = [
        migrations.CreateModel(
            name='SkillTotals',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('total_sp', models.BigIntegerField()),
                ('unallocated_sp', models.IntegerField()),
                ('character', models.OneToOneField(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CharacterAudit')),
            ],
        ),
    ]



# Generated by Django 3.1.1 on 2021-05-31 10:25

import corptools.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0046_auto_20210427_0734'),
    ]

    operations = [
        migrations.AlterField(
            model_name='skilllist',
            name='skill_list',
            field=models.TextField(default='', null=True, validators=[
                                   corptools.validators.valid_json]),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-13 04:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0049_alter_mailmessage_is_read'),
    ]

    operations = [
        migrations.AddField(
            model_name='mailmessage',
            name='from_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.evename'),
        ),
        migrations.AddField(
            model_name='mailrecipient',
            name='recipient_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.evename'),
        ),
    ]



"""
Alliance Auth Test Suite Django settings.
"""

from allianceauth.project_template.project_name.settings.base import *


# Celery configuration
CELERY_ALWAYS_EAGER = True  # Forces celery to run locally for testing

INSTALLED_APPS += [
    'corptools',
    'networkx'
]

ROOT_URLCONF = 'tests.urls'

NOSE_ARGS = [
    # '--with-coverage',
    # '--cover-package=',
    # '--exe',  # If your tests need this to be found/run, check they py files are not chmodded +x
]

CACHES['default'] = {'BACKEND': 'django.core.cache.backends.db.DatabaseCache'}


PASSWORD_HASHERS = [
    'django.contrib.auth.hashers.MD5PasswordHasher',
]

# LOGGING = None  # Comment out to enable logging for debugging

# Register an application at https://developers.eveonline.com for Authentication
# & API Access and fill out these settings. Be sure to set the callback URL
# to https://example.com/sso/callback substituting your domain for example.com
# Logging in to auth requires the publicData scope (can be overridden through the
# LOGIN_TOKEN_SCOPES setting). Other apps may require more (see their docs).
ESI_SSO_CLIENT_ID = '123'
ESI_SSO_CLIENT_SECRET = '123'
ESI_SSO_CALLBACK_URL = '123'



# Generated by Django 3.2.5 on 2021-08-08 05:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0056_auto_20210808_0414'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='titlefilter',
            options={'verbose_name': 'Smart Filter: Corporate Title checks',
                     'verbose_name_plural': 'Smart Filter: Corporate Title checks'},
        ),
    ]



import os

from bravado.exception import HTTPError
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required, user_passes_test
from django.core.exceptions import PermissionDenied
from django.db import IntegrityError
from django.db.models import Count
from django.shortcuts import render, redirect, get_object_or_404
from django.utils.translation import ugettext_lazy as _
from esi.decorators import token_required
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from django.http import HttpResponse
import xml.etree.ElementTree as ET
import csv
import re
import json
from itertools import chain
from .models import *
from .tasks import update_character, update_all_characters, update_ore_comp_table, update_or_create_map, process_ores_from_esi, update_all_corps, check_account
from .forms import UploadForm
from . import app_settings
from django_celery_beat.models import CrontabSchedule, PeriodicTask

CORP_REQUIRED_SCOPES = [
    'esi-assets.read_corporation_assets.v1',
    'esi-characters.read_corporation_roles.v1',
    'esi-corporations.read_corporation_membership.v1',
    'esi-corporations.read_divisions.v1',
    'esi-corporations.read_starbases.v1',
    'esi-corporations.read_structures.v1',
    'esi-corporations.read_titles.v1',
    'esi-corporations.track_members.v1',
    'esi-industry.read_corporation_jobs.v1',
    'esi-industry.read_corporation_mining.v1',
    'esi-killmails.read_corporation_killmails.v1',
    'esi-markets.read_corporation_orders.v1',
    'esi-planets.read_customs_offices.v1',
    'esi-search.search_structures.v1',
    'esi-universe.read_structures.v1',
    'esi-wallet.read_corporation_wallets.v1'
]


@login_required
@token_required(scopes=app_settings.get_character_scopes())
def add_char(request, token):
    CharacterAudit.objects.update_or_create(
        character=EveCharacter.objects.get_character_by_id(token.character_id))
    update_character.apply_async(args=[token.character_id], priority=6)
    return redirect('corptools:view')


@login_required
@token_required(scopes=CORP_REQUIRED_SCOPES)
def add_corp(request, token):
    char = EveCharacter.objects.get_character_by_id(token.character_id)
    corp, created = EveCorporationInfo.objects.get_or_create(corporation_id=char.corporation_id,
                                                             defaults={'member_count': 0,
                                                                       'corporation_ticker': char.corporation_ticker,
                                                                       'corporation_name': char.corporation_name
                                                                       })
    CorporationAudit.objects.update_or_create(corporation=corp)
    update_all_corps.apply_async(priority=6)
    return redirect('corptools:corp_menu')


@login_required
@permission_required('corptools.view_characteraudit')
def corptools_menu(request):
    # get available models
    cas = CharacterAudit.objects.visible_to(request.user)\
        .select_related('character__character_ownership__user__profile__main_character',
                        'character__characteraudit')\
        .prefetch_related('character__character_ownership__user__character_ownerships')\
        .prefetch_related('character__character_ownership__user__character_ownerships__character')\
        .prefetch_related('character__character_ownership__user__character_ownerships__character__characteraudit')\


    chars = {}
    orphans = []
    for char in cas:
        try:
            main = char.character.character_ownership.user.profile.main_character
            if main:
                if main.character_name not in chars:
                    chars[str(main.character_id)] = {'main': main,
                                                     'audit': char}
            else:
                orphans.append(char)
        except ObjectDoesNotExist:
            orphans.append(char)

    if len(chars) == 1:
        return redirect('corptools:overview', chars[list(chars.keys())[0]]['main'].character_id)

    return render(request, 'corptools/menu.html', context={'characters': chars, 'orphans': orphans})


@login_required
@permission_required('corptools.admin')
def admin(request):
    # get available models
    names = EveName.objects.all().count()
    types = EveItemType.objects.all().count()
    dogma = EveItemDogmaAttribute.objects.all().count()
    groups = EveItemGroup.objects.all().count()
    categorys = EveItemCategory.objects.all().count()
    type_mets = InvTypeMaterials.objects.count()
    regions = MapRegion.objects.all().count()
    constellations = MapConstellation.objects.all().count()
    systems = MapSystem.objects.all().count()
    gates = MapSystemGate.objects.all().count()
    location = EveLocation.objects.all().count()
    bridges = MapJumpBridge.objects.all().count()

    characters = CharacterAudit.objects.all().count()
    skilllists = SkillList.objects.all().count()
    corpations = CorporationAudit.objects.all().count()

    char_tasks = PeriodicTask.objects.filter(
        task='corptools.tasks.update_subset_of_characters', enabled=True).count()
    corp_tasks = PeriodicTask.objects.filter(
        task='corptools.tasks.update_all_corps', enabled=True).count()

    context = {
        "names": names,
        "types": types,
        "dogma": dogma,
        "groups": groups,
        "categorys": categorys,
        "characters": characters,
        "skilllists": skilllists,
        "corpations": corpations,
        "type_mets": type_mets,
        "regions": regions,
        "constellations": constellations,
        "systems": systems,
        "location": location,
        "bridges": bridges,
        "gates": gates,
        "char_tasks": char_tasks,
        "corp_tasks": corp_tasks,
        "form": UploadForm(),
    }

    return render(request, 'corptools/admin.html', context=context)


@login_required
@permission_required('corptools.admin')
def admin_run_tasks(request):
    if request.method == 'POST':
        if request.POST.get('run_universe'):
            update_or_create_map.apply_async(priority=6)
        if request.POST.get('run_update_all'):
            update_all_characters.apply_async(priority=6)
        if request.POST.get('run_update_eve_models'):
            update_ore_comp_table.apply_async(priority=6)
            process_ores_from_esi.apply_async(priority=6)
        if request.POST.get('run_corp_updates'):
            update_all_corps.apply_async(priority=6)
    return redirect('corptools:admin')


@login_required
@permission_required('corptools.admin')
def admin_create_tasks(request):
    schedule_char, _ = CrontabSchedule.objects.get_or_create(minute='15,45',
                                                             hour='*',
                                                             day_of_week='*',
                                                             day_of_month='*',
                                                             month_of_year='*',
                                                             timezone='UTC'
                                                             )

    schedule_corp, _ = CrontabSchedule.objects.get_or_create(minute='30',
                                                             hour='12',
                                                             day_of_week='*',
                                                             day_of_month='*',
                                                             month_of_year='*',
                                                             timezone='UTC'
                                                             )

    task_char = PeriodicTask.objects.update_or_create(
        task='corptools.tasks.update_subset_of_characters',
        defaults={
            'crontab': schedule_char,
            'name': 'Character Audit Rolling Update',
            'enabled': True
        }
    )
    # if created:
    #    messages.info(request, "Created Character Task")
    # else:
    #    messages.info(request, "Reset Character Task to defaults")

    task_corp = PeriodicTask.objects.update_or_create(
        task='corptools.tasks.update_all_corps',
        defaults={
            'crontab': schedule_corp,
            'name': 'Corporation Audit Update',
            'enabled': True
        }
    )
    # if created:
    #    messages.info(request, "Created Corporation Task")
    # else:
    #    messages.info(request, "Reset Corporation Task to defaults")

    # https://github.com/celery/django-celery-beat/issues/106
    messages.info(
        request, "Created/Reset Character and Corporation Task to defaults")

    return redirect('corptools:admin')


@login_required
@permission_required('corptools.admin')
def admin_add_pyfa_xml(request):
    if request.method == 'POST':
        form = UploadForm(request.POST, request.FILES)
        if form.is_valid():
            xml_file = form.files.get('file')
            if xml_file.content_type == 'text/xml':
                skills = {}
                tree = ET.parse(xml_file)
                root = tree.getroot()
                if root.tag == "plan":
                    for child in root:
                        if child.tag == "entry":
                            skill = child.attrib.get('skill')
                            level = child.attrib.get('level')
                            if skill in skills:
                                if level > skills[skill]:
                                    skills[skill] = level
                            else:
                                skills[skill] = level
                sl, created = SkillList.objects.update_or_create(name=request.POST["name"],
                                                                 defaults={
                    "skill_list": json.dumps(skills)
                })

                messages.success(request, "File Uploaded and Processed! {}: {}".format(
                    ("Created" if created else "Updated"),
                    request.POST["name"]
                ))
            else:
                messages.error(
                    request, "File Upload Failed! Must be XML Exported from PYFA!")
        else:
            messages.error(request, "File Upload Failed! Invalid Form")
    else:
        messages.error(
            request, "File Upload Failed! What are you doing your not meant to be here?")

    return redirect('corptools:admin')


@login_required
@permission_required('corptools.admin')
def skill_list_editor(request):
    pass


@login_required
def update_account(request, character_id):
    check_account.apply_async(args=[character_id], priority=6)
    messages.success(request, "Requested an update task.")
    return redirect('corptools:view')



# Generated by Django 2.2.12 on 2020-07-27 00:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0016_auto_20200622_0643'),
    ]

    operations = [
        migrations.AlterField(
            model_name='characterasset',
            name='type_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveItemType'),
        ),
        migrations.AlterField(
            model_name='corpasset',
            name='type_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveItemType'),
        ),
        migrations.CreateModel(
            name='EveLocation',
            fields=[
                ('location_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('location_name', models.CharField(max_length=255)),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('system', models.ForeignKey(default=None, null=True,
                                             on_delete=django.db.models.deletion.SET_NULL, to='corptools.MapSystem')),
            ],
        ),
        migrations.AddField(
            model_name='characterasset',
            name='location_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveLocation'),
        ),
        migrations.AddField(
            model_name='corpasset',
            name='location_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveLocation'),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError

from corptools.tasks import update_or_create_map, process_ores_from_esi, update_ore_comp_table
from corptools.models import OreTaxRates


class Command(BaseCommand):
    help = 'Update or Populate base EVE Models'

    def add_arguments(self, parser):
        parser.add_argument('--inline', action='store_true',
                            help='Run update in this Console not via Celery')

    def handle(self, *args, **options):
        self.stdout.write("Confirming DB models!")
        if OreTaxRates.objects.all().count() == 0:
            OreTaxRates.objects.create(
                tag="Ore Tax",
                ore_rate=0.2,
                ubiquitous_rate=0.2,
                common_rate=0.2,
                uncommon_rate=0.2,
                rare_rate=0.2,
                excptional_rate=0.2
            )

        if options['inline']:
            self.stdout.write("Running Tasks inline this may take some time!")
            self.stdout.write("Starting Map Update")
            update_or_create_map()
            self.stdout.write("Starting Asteroid Data Update")
            process_ores_from_esi()
            self.stdout.write("Starting Ore Comp Update")
            update_ore_comp_table()
            self.stdout.write("Done Tasks!")
        else:
            self.stdout.write("Sending Tasks to celery for processing!")
            self.stdout.write("Sending Map Update Task")
            update_or_create_map.apply_async(priority=6)
            self.stdout.write("Sending Asteroid Data Task")
            process_ores_from_esi.apply_async(priority=6)
            self.stdout.write("Sending Ore Tasks")
            update_ore_comp_table.apply_async(priority=6)
            self.stdout.write("Done Tasks!")



# Generated by Django 2.2.12 on 2020-08-14 10:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0026_auto_20200811_0846'),
    ]

    operations = [
        migrations.CreateModel(
            name='SkillListFile',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, max_length=255)),
                ('skilllist', models.FileField(upload_to='skill_lists/')),
                ('uploaded_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



# Generated by Django 3.1.1 on 2020-10-17 06:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0035_auto_20200929_0114'),
    ]

    operations = [
        migrations.AddField(
            model_name='characteraudit',
            name='cache_expire_notif',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_notif',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('notification_id', models.BigIntegerField()),
                ('sender_id', models.IntegerField()),
                ('sender_type', models.CharField(choices=[('character', 'character'), ('corporation', 'corporation'), (
                    'alliance', 'alliance'), ('faction', 'faction'), ('other', 'other')], max_length=15)),
                ('notification_text', models.TextField(default=None, null=True)),
                ('timestamp', models.DateTimeField()),
                ('notification_type', models.CharField(max_length=50)),
                ('is_read', models.NullBooleanField(default=None)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
            ],
        ),
        migrations.AddIndex(
            model_name='notification',
            index=models.Index(
                fields=['notification_id'], name='corptools_n_notific_77c7f2_idx'),
        ),
        migrations.AddIndex(
            model_name='notification',
            index=models.Index(fields=['timestamp'],
                               name='corptools_n_timesta_ca02fe_idx'),
        ),
        migrations.AddIndex(
            model_name='notification',
            index=models.Index(
                fields=['notification_type'], name='corptools_n_notific_8ff99b_idx'),
        ),
    ]



from unittest import mock
from django.test import TestCase
from allianceauth.tests.auth_utils import AuthUtils
from ..models import CharacterAudit
from django.contrib.auth.models import Permission

from allianceauth.eveonline.models import EveCharacter
from allianceauth.authentication.models import CharacterOwnership


class TestCorptoolsAccessPerms(TestCase):
    def setUp(cls):
        cls.user1 = AuthUtils.create_user('Main User1')
        cls.user2 = AuthUtils.create_user('Main User2')
        cls.user3 = AuthUtils.create_user('Main User3')

        char1 = EveCharacter.objects.create(character_id=1,
                                            character_name='character.name1',
                                            corporation_id=123,
                                            corporation_name='corporation.name1',
                                            corporation_ticker='ABC')
        cls.ca1 = CharacterAudit.objects.create(character=char1)

        cls.user1.profile.main_character = char1
        CharacterOwnership.objects.create(
            user=cls.user1, character=char1, owner_hash="abc123")
        cls.user1.profile.save()

        char2 = EveCharacter.objects.create(character_id=2,
                                            character_name='character.name2',
                                            corporation_id=123,
                                            corporation_name='corporation.name1',
                                            corporation_ticker='ABC')
        cls.ca2 = CharacterAudit.objects.create(character=char2)
        CharacterOwnership.objects.create(
            user=cls.user1, character=char2, owner_hash="cba123")
        cls.user1.profile.refresh_from_db()

        char3 = EveCharacter.objects.create(character_id=3,
                                            character_name='character.name3',
                                            corporation_id=2,
                                            corporation_name='corporation.name2',
                                            corporation_ticker='ABC2',
                                            alliance_id=3,
                                            alliance_name='test alliance2',
                                            alliance_ticker='TEST')
        cls.ca3 = CharacterAudit.objects.create(character=char3)

        cls.user2.profile.main_character = char3
        CharacterOwnership.objects.create(
            user=cls.user2, character=char3, owner_hash="cba321")

        cls.user2.profile.save()
        cls.user2.profile.refresh_from_db()

        char4 = EveCharacter.objects.create(character_id=4,
                                            character_name='character.name4',
                                            corporation_id=2,
                                            corporation_name='corporation.name2',
                                            corporation_ticker='ABC',
                                            alliance_id=3,
                                            alliance_name='test alliance2',
                                            alliance_ticker='TEST')
        cls.ca4 = CharacterAudit.objects.create(character=char4)

        char5 = EveCharacter.objects.create(character_id=5,
                                            character_name='character.name5',
                                            corporation_id=3,
                                            corporation_name='corporation.name3',
                                            corporation_ticker='ABC3',
                                            alliance_id=4,
                                            alliance_name='test alliance2',
                                            alliance_ticker='TEST')
        cls.ca5 = CharacterAudit.objects.create(character=char5)

        cls.user3.profile.main_character = char5
        CharacterOwnership.objects.create(
            user=cls.user3, character=char5, owner_hash="abc432")

        cls.user3.profile.save()
        cls.user3.profile.refresh_from_db()

        char6 = EveCharacter.objects.create(character_id=6,
                                            character_name='character.name6',
                                            corporation_id=3,
                                            corporation_name='corporation.name3',
                                            corporation_ticker='ABC3',
                                            alliance_id=4,
                                            alliance_name='test alliance2',
                                            alliance_ticker='TEST2')
        cls.ca6 = CharacterAudit.objects.create(character=char6)

        char7 = EveCharacter.objects.create(character_id=7,
                                            character_name='character.name7',
                                            corporation_id=4,
                                            corporation_name='corporation.name4',
                                            corporation_ticker='ABC4',
                                            alliance_id=4,
                                            alliance_name='test alliance2',
                                            alliance_ticker='TEST2')
        cls.ca7 = CharacterAudit.objects.create(character=char7)
        CharacterOwnership.objects.create(
            user=cls.user3, character=char7, owner_hash="def432")

        char8 = EveCharacter.objects.create(character_id=8,
                                            character_name='character.name8',
                                            corporation_id=5,
                                            corporation_name='corporation.name4',
                                            corporation_ticker='ABC4',
                                            alliance_id=3,
                                            alliance_name='test alliance3',
                                            alliance_ticker='TEST3')
        cls.ca8 = CharacterAudit.objects.create(character=char8)

        cls.view_corp_permission = Permission.objects.get_by_natural_key(
            'corp_hr', 'corptools', 'characteraudit')
        cls.view_alliance_permission = Permission.objects.get_by_natural_key(
            'alliance_hr', 'corptools', 'characteraudit')
        cls.view_all_permission = Permission.objects.get_by_natural_key(
            'global_hr', 'corptools', 'characteraudit')

    def test_no_perms_get_self_u1(self):  # always get self.
        cs = CharacterAudit.objects.visible_to(self.user1)
        self.assertIn(self.ca1, cs)
        self.assertIn(self.ca2, cs)
        self.assertNotIn(self.ca3, cs)
        self.assertNotIn(self.ca4, cs)
        self.assertNotIn(self.ca5, cs)
        self.assertNotIn(self.ca6, cs)
        self.assertNotIn(self.ca7, cs)
        self.assertNotIn(self.ca8, cs)

    def test_no_perms_get_self_u2(self):  # always get self.
        cs = CharacterAudit.objects.visible_to(self.user2)
        self.assertNotIn(self.ca1, cs)
        self.assertNotIn(self.ca2, cs)
        self.assertIn(self.ca3, cs)
        self.assertNotIn(self.ca4, cs)
        self.assertNotIn(self.ca5, cs)
        self.assertNotIn(self.ca6, cs)
        self.assertNotIn(self.ca7, cs)
        self.assertNotIn(self.ca8, cs)

    def test_no_perms_get_self_u3(self):  # always get self.
        cs = CharacterAudit.objects.visible_to(self.user3)
        self.assertNotIn(self.ca1, cs)
        self.assertNotIn(self.ca2, cs)
        self.assertNotIn(self.ca3, cs)
        self.assertNotIn(self.ca4, cs)
        self.assertIn(self.ca5, cs)
        self.assertNotIn(self.ca6, cs)
        self.assertIn(self.ca7, cs)
        self.assertNotIn(self.ca8, cs)

    def test_no_perms_get_self_in_alliance(self):
        cs = CharacterAudit.objects.visible_to(self.user2)
        self.assertNotIn(self.ca1, cs)
        self.assertNotIn(self.ca2, cs)
        self.assertIn(self.ca3, cs)
        self.assertNotIn(self.ca4, cs)
        self.assertNotIn(self.ca5, cs)
        self.assertNotIn(self.ca6, cs)
        self.assertNotIn(self.ca7, cs)
        self.assertNotIn(self.ca8, cs)

    def test_get_corp_in_alliance(self):
        self.user2.user_permissions.add(self.view_corp_permission)
        cs = CharacterAudit.objects.visible_to(self.user2)
        self.assertNotIn(self.ca1, cs)
        self.assertNotIn(self.ca2, cs)
        self.assertIn(self.ca3, cs)
        self.assertIn(self.ca4, cs)
        self.assertNotIn(self.ca5, cs)
        self.assertNotIn(self.ca6, cs)
        self.assertNotIn(self.ca7, cs)
        self.assertNotIn(self.ca8, cs)

    def test_get_alliance(self):
        self.user3.user_permissions.add(self.view_alliance_permission)
        cs = CharacterAudit.objects.visible_to(self.user3)
        self.assertNotIn(self.ca1, cs)
        self.assertNotIn(self.ca2, cs)
        self.assertNotIn(self.ca3, cs)
        self.assertNotIn(self.ca4, cs)
        self.assertIn(self.ca5, cs)
        self.assertIn(self.ca6, cs)
        self.assertIn(self.ca7, cs)
        self.assertNotIn(self.ca8, cs)

    def test_global_perms_u1(self):
        self.user3.user_permissions.add(self.view_all_permission)
        cs = CharacterAudit.objects.visible_to(self.user3)
        self.assertIn(self.ca1, cs)
        self.assertIn(self.ca2, cs)
        self.assertIn(self.ca3, cs)
        self.assertIn(self.ca4, cs)
        self.assertIn(self.ca5, cs)
        self.assertIn(self.ca6, cs)
        self.assertIn(self.ca7, cs)
        self.assertIn(self.ca8, cs)

    def test_global_perms_u2(self):
        self.user2.user_permissions.add(self.view_all_permission)
        cs = CharacterAudit.objects.visible_to(self.user2)
        self.assertIn(self.ca1, cs)
        self.assertIn(self.ca2, cs)
        self.assertIn(self.ca3, cs)
        self.assertIn(self.ca4, cs)
        self.assertIn(self.ca5, cs)
        self.assertIn(self.ca6, cs)
        self.assertIn(self.ca7, cs)
        self.assertIn(self.ca8, cs)

    def test_global_perms_u3(self):
        self.user2.user_permissions.add(self.view_all_permission)
        cs = CharacterAudit.objects.visible_to(self.user2)
        self.assertIn(self.ca1, cs)
        self.assertIn(self.ca2, cs)
        self.assertIn(self.ca3, cs)
        self.assertIn(self.ca4, cs)
        self.assertIn(self.ca5, cs)
        self.assertIn(self.ca6, cs)
        self.assertIn(self.ca7, cs)
        self.assertIn(self.ca8, cs)



import logging
from celery import shared_task
from ..models import CorporationWalletJournalEntry, CorporationAudit, CorporationWalletDivision, EveName

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo

from esi.models import Token
from django.utils import timezone
from django.db.models import Q
from allianceauth.services.tasks import QueueOnce
from bravado.exception import HTTPForbidden

from .. import providers

import bz2
import re
import requests
import datetime

logger = logging.getLogger(__name__)


def get_corp_token(corp_id, scopes, req_roles):
    """
    Helper method to get a token for a specific character from a specific corp with specific scopes
    :param corp_id: Corp to filter on.
    :param scopes: array of ESI scope strings to search for.
    :param req_roles: roles required on the character.
    :return: :class:esi.models.Token or False
    """
    if 'esi-characters.read_corporation_roles.v1' not in scopes:
        scopes.append("esi-characters.read_corporation_roles.v1")

    char_ids = EveCharacter.objects.filter(
        corporation_id=corp_id).values('character_id')
    tokens = Token.objects \
        .filter(character_id__in=char_ids) \
        .require_scopes(scopes)

    for token in tokens:
        roles = providers.esi.client.Character.get_characters_character_id_roles(character_id=token.character_id,
                                                                                 token=token.valid_access_token()).result()
        has_roles = False
        for role in roles.get('roles', []):
            if role in req_roles:
                has_roles = True

        if has_roles:
            return token
        else:
            pass  # TODO Maybe remove token?

    return False


# pagnated results
def update_corp_wallet_journal(corp_id, wallet_division, full_update=False):
    audit_corp = CorporationAudit.objects.get(
        corporation__corporation_id=corp_id)
    division = CorporationWalletDivision.objects.get(
        corporation=audit_corp, division=wallet_division)
    logger.debug("Updating wallet transactions for: {} (Div: {})".format(
        audit_corp.corporation.corporation_name, division))

    req_scopes = ['esi-wallet.read_corporation_wallets.v1',
                  'esi-characters.read_corporation_roles.v1']
    req_roles = ['CEO', 'Director', 'Accountant', 'Junior_Accountant']

    token = get_corp_token(corp_id, req_scopes, req_roles)

    if not token:
        return "No Tokens"

    journal_items = providers.esi.client.Wallet.get_corporations_corporation_id_wallets_division_journal(corporation_id=audit_corp.corporation.corporation_id,
                                                                                                         division=wallet_division,
                                                                                                         token=token.valid_access_token()).results()

    _current_journal = CorporationWalletJournalEntry.objects.filter(
        division=division).values_list('entry_id', flat=True)  # TODO add time filter
    _current_eve_ids = list(
        EveName.objects.all().values_list('eve_id', flat=True))

    _new_names = []

    items = []
    for item in journal_items:
        if item.get('id') not in _current_journal:
            if item.get('second_party_id') not in _current_eve_ids:
                _new_names.append(item.get('second_party_id'))
                _current_eve_ids.append(item.get('second_party_id'))
            if item.get('first_party_id') not in _current_eve_ids:
                _new_names.append(item.get('first_party_id'))
                _current_eve_ids.append(item.get('first_party_id'))

            wallet_item = CorporationWalletJournalEntry(division=division,
                                                        amount=item.get(
                                                            'amount'),
                                                        balance=item.get(
                                                            'balance'),
                                                        context_id=item.get(
                                                            'context_id'),
                                                        context_id_type=item.get(
                                                            'context_id_type'),
                                                        date=item.get('date'),
                                                        description=item.get(
                                                            'description'),
                                                        first_party_id=item.get(
                                                            'first_party_id'),
                                                        first_party_name_id=item.get(
                                                            'first_party_id'),
                                                        entry_id=item.get(
                                                            'id'),
                                                        reason=item.get(
                                                            'reason'),
                                                        ref_type=item.get(
                                                            'ref_type'),
                                                        second_party_id=item.get(
                                                            'second_party_id'),
                                                        second_party_name_id=item.get(
                                                            'second_party_id'),
                                                        tax=item.get('tax'),
                                                        tax_receiver_id=item.get(
                                                            'tax_receiver_id'),
                                                        )
            items.append(wallet_item)

    created_names = EveName.objects.create_bulk_from_esi(_new_names)

    if created_names:
        CorporationWalletJournalEntry.objects.bulk_create(items)
    else:
        raise Exception("DB Fail")


def update_corp_wallet_division(corp_id, full_update=False):  # pagnated results
    # logger.debug("Started wallet divs for: %s" % str(character_id))
    audit_corp = CorporationAudit.objects.get(
        corporation__corporation_id=corp_id)

    req_scopes = ['esi-wallet.read_corporation_wallets.v1',
                  'esi-characters.read_corporation_roles.v1', 'esi-corporations.read_divisions.v1']
    req_roles = ['CEO', 'Director']

    token = get_corp_token(corp_id, req_scopes, req_roles)

    if not token:
        return "No Tokens"
    division_names = providers.esi.client.Corporation.get_corporations_corporation_id_divisions(corporation_id=audit_corp.corporation.corporation_id,
                                                                                                token=token.valid_access_token()).result()
    names = {}
    for division in division_names.get('wallet'):
        names[division.get('division')] = division.get('name')

    divisions = providers.esi.client.Wallet.get_corporations_corporation_id_wallets(corporation_id=audit_corp.corporation.corporation_id,
                                                                                    token=token.valid_access_token()).result()

    for division in divisions:
        _division_item, _created = CorporationWalletDivision.objects \
            .update_or_create(corporation=audit_corp, division=division.get('division'),
                              defaults={'balance': division.get('balance'),
                                        'name': names[division.get('division')]})

        if _division_item:
            update_corp_wallet_journal(corp_id, division.get('division'),
                                       full_update=full_update)  # inline not async

    audit_corp.last_update_wallet = timezone.now()
    audit_corp.save()

    return "Finished wallet divs for: {0}".format(audit_corp.corporation.corporation_name)



# Generated by Django 3.1.1 on 2021-01-25 07:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0042_remove_skilllist_eft'),
    ]

    operations = [
        migrations.AddField(
            model_name='corporationaudit',
            name='cache_expire_wallet',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='corporationaudit',
            name='last_update_wallet',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
    ]



import os

from bravado.exception import HTTPError
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required, user_passes_test
from django.core.exceptions import PermissionDenied
from django.db import IntegrityError
from django.db.models import Count, F, Sum, Max, Q, FloatField
from django.db.models.query import prefetch_related_objects
from django.shortcuts import render, redirect, get_object_or_404
from django.utils.translation import ugettext_lazy as _
from esi.decorators import token_required
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from django.http import HttpResponse
import csv
import re
import copy

from ..models import *
from .. import providers


def get_alts(request, character_id):
    if character_id is None:
        main_char = request.user.profile.main_character
    else:
        main_char = EveCharacter.objects\
            .select_related('character_ownership', 'character_ownership__user__profile', 'character_ownership__user__profile__main_character', )\
            .get(character_id=int(character_id))\
            .character_ownership.user.profile.main_character

    linked_characters = main_char.character_ownership.user.character_ownerships.all(
    ).values_list('character_id', flat=True)
    # check access
    visible = CharacterAudit.objects.visible_to(
        request.user).values_list('character_id', flat=True)
    if main_char.id not in visible:
        account_chars = request.user.profile.main_character.character_ownership.user.character_ownerships.all(
        ).values_list('character_id', flat=True)
        logger.warning(
            f"{request.user} Can See {list(visible)}, requested {main_char.id}")
        if main_char.id in account_chars:
            pass
        else:
            raise PermissionDenied(
                "You do not have access to view this character")

    if not request.user.has_perm('corptools.view_characteraudit'):
        logger.warning(
            f"{request.user} doesnot have Perm requested, Requested {main_char.id}")
        raise PermissionDenied("You do not have access to view this character")

    character_id = main_char.character_id
    characters = EveCharacter.objects\
        .select_related('character_ownership__user', 'character_ownership__character')\
        .prefetch_related('character_ownership')\
        .filter(id__in=linked_characters).select_related('characteraudit')

    net_worth = 0
    for character in characters:
        try:
            net_worth += character.characteraudit.balance
        except:
            pass
    return main_char, characters, net_worth


@login_required
def assets(request, character_id=None):
    main_char, characters, net_worth = get_alts(request, character_id)
    asset_locations = {0: "Everywhere"}

    asset_locs = CharacterAsset.objects.filter(character__character__character_id__in=characters.values_list('character_id', flat=True),
                                               location_name__isnull=False).values_list('location_name').distinct()
    asset_locs = EveLocation.objects.filter(
        location_id__in=asset_locs).order_by('location_name')

    for loc in asset_locs:
        asset_locations[loc.location_id] = loc.location_name

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "asset_locations": asset_locations
    }

    return render(request, 'corptools/character/assets.html', context=context)


@login_required
def assets_lists(request, character_id=None, location_id=None):
    # get available models
    main_char, characters, net_worth = get_alts(request, character_id)

    capital_groups = [30, 547, 659, 1538, 485, 902, 513, 883]
    subcap_cat = [6]
    noteable_cats = [4, 20, 23, 25, 34, 35, 87, 91]
    structure_cats = [22, 24, 40, 41, 46, 65, 66, ]
    bpo_cats = [9]

    assets = CharacterAsset.objects\
        .filter((Q(blueprint_copy=None) | Q(blueprint_copy=False)),
                character__character__character_id__in=characters.values_list('character_id', flat=True))

    if location_id != '0':
        asset_locations = assets.filter(
            location_name_id=int(location_id)).values_list('item_id')
        assets = assets.filter(Q(location_name_id=int(location_id)) | Q(
            location_id__in=asset_locations))

    assets = assets.values('type_name__group__group_id')\
        .annotate(grp_total=Sum('quantity'))\
        .annotate(grp_name=F('type_name__group__name'))\
        .annotate(grp_id=F('type_name__group_id'))\
        .annotate(cat_id=F('type_name__group__category_id'))\
        .order_by('-grp_total')

    capital_asset_groups = []
    subcap_asset_groups = []
    noteable_asset_groups = []
    structure_asset_groups = []
    bpo_asset_groups = []
    remaining_asset_groups = []

    for grp in assets:
        if grp['grp_id'] in capital_groups:
            capital_asset_groups.append(grp)
        elif grp['cat_id'] in subcap_cat:
            subcap_asset_groups.append(grp)
        elif grp['cat_id'] in noteable_cats:
            noteable_asset_groups.append(grp)
        elif grp['cat_id'] in structure_cats:
            structure_asset_groups.append(grp)
        elif grp['cat_id'] in bpo_cats:
            bpo_asset_groups.append(grp)
        else:
            remaining_asset_groups.append(grp)

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "capital_asset_groups": capital_asset_groups,
        "noteable_asset_groups": noteable_asset_groups,
        "subcap_asset_groups": subcap_asset_groups,
        "structure_asset_groups": structure_asset_groups,
        "bpo_asset_groups": bpo_asset_groups,
        "remaining_asset_groups": remaining_asset_groups
    }

    return render(request, 'corptools/character/assets_lists.html', context=context)


@login_required
def wallet(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)

    wallet_journal = CharacterWalletJournalEntry.objects\
        .filter(character__character__character_id__in=characters.values_list('character_id', flat=True))\
        .select_related('first_party_name', 'second_party_name', 'character__character').order_by('-date')[:5000]

    graph_data_model = {"0000": {}, "0100": {}, "0200": {}, "0300": {}, "0400": {}, "0500": {}, "0600": {}, "0700": {}, "0800": {}, "0900": {}, "1000": {
    }, "1100": {}, "1200": {}, "1300": {}, "1400": {}, "1500": {}, "1600": {}, "1700": {}, "1800": {}, "1900": {}, "2000": {}, "2100": {}, "2200": {}, "2300": {}}
    # %-H
    graph_people = []
    chord_data = {}
    transactions = []
    for wd in wallet_journal:
        dt = wd.date.strftime("%H") + "00"
        cn = wd.character.character.character_name
        if cn not in graph_people:
            graph_people.append(cn)
        if cn not in graph_data_model[dt]:
            graph_data_model[dt][cn] = 0
        graph_data_model[dt][cn] += 1

        wallet_type_tracking = [
            'corporation_account_withdrawal', 'player_trading', 'player_donation']
        if wd.ref_type in wallet_type_tracking:
            if wd.entry_id not in transactions:
                if wd.amount > 0:
                    fp = wd.first_party_name.name
                    sp = wd.second_party_name.name
                    am = wd.amount
                else:
                    sp = wd.first_party_name.name
                    fp = wd.second_party_name.name
                    am = wd.amount * -1

                if fp not in chord_data:
                    chord_data[fp] = {}
                if sp not in chord_data[fp]:
                    chord_data[fp][sp] = {"a": 0, "c": 0}

                transactions.append(wd.entry_id)
                chord_data[fp][sp]["a"] += am
                chord_data[fp][sp]["c"] += 1

    has_perms = request.user.has_perm('corptools.global_hr') or \
        request.user.has_perm('corptools.alliance_hr') or \
        request.user.has_perm('corptools.corp_hr')
    context = {
        "manager_perms": has_perms,
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "wallet": wallet_journal,
        "graphs": graph_data_model,
        "graph_people": graph_people,
        "chords": chord_data,
    }

    return render(request, 'corptools/character/wallet.html', context=context)


@login_required
def status(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)
    character_ids = characters.values_list('character_id', flat=True)
    char_skill_total = Skill.objects\
        .filter(character__character__character_id__in=character_ids)\
        .values('character')\
        .annotate(char=F('character__character__character_name'))\
        .annotate(total_sp=Sum('skillpoints_in_skill'))

    bios = None

    table_data = {}
    for char in characters:
        table_data[char.character_name] = {
            "character": char, "history": {}, "bio": "Nothing of value is here"}

    for c in char_skill_total:
        table_data[c.get('char')]["total_sp"] = c.get('total_sp')

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "table_data": table_data,
    }

    return render(request, 'corptools/character/status.html', context=context)


@login_required
def pub_data(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)
    character_ids = characters.values_list('character_id', flat=True)
    corp_histories = CorporationHistory.objects\
        .filter(character__character__character_id__in=character_ids)\
        .select_related('character__character', 'corporation_name')
    char_skill_total = Skill.objects\
        .filter(character__character__character_id__in=character_ids)\
        .values('character')\
        .annotate(char=F('character__character__character_name'))\
        .annotate(total_sp=Sum('skillpoints_in_skill'))

    bios = None

    table_data = {}
    for char in characters:
        table_data[char.character_name] = {
            "character": char, "history": {}, "bio": "Nothing of value is here"}

    for h in corp_histories:
        table_data[h.character.character.character_name]["history"][str(h.record_id)] = {
            "name": h.corporation_name.name, "id": h.corporation_id, "started": h.start_date, "open": h.is_deleted}

    for c in char_skill_total:
        table_data[c.get('char')]["total_sp"] = c.get('total_sp')

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "table_data": table_data,
    }

    return render(request, 'corptools/character/public.html', context=context)


@login_required
def skills(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)

    user = main_char.character_ownership.user.id
    skills_list = providers.skills.get_and_cache_user(user)

    skill_tables = skills_list.get("skills_list")

    queues = SkillQueue.objects.filter(character__character__in=characters)\
        .select_related('skill_name', 'character__character')
    totals = SkillTotals.objects.filter(character__character__in=characters)\
        .select_related('character__character')

    for que in queues:
        char = que.character.character.character_name
        if char not in skill_tables:
            skill_tables[char] = {"character": que.character,
                                  "omega": True, "skills": {}, "queue": []}
        skill_tables[char]["queue"].append(
            {
                "queue_position": que.queue_position,
                "skill_name": que.skill_name.name,
                "finish_date": que.finish_date,
                "finish_level": que.finish_level
            }
        )

    for total in totals:
        char = total.character.character.character_name
        if char not in skill_tables:
            skill_tables[char] = {"character": total.character,
                                  "omega": True, "skills": {}, "queue": []}
        skill_tables[char]["total_sp"] = total.total_sp
        skill_tables[char]["unallocated_sp"] = total.unallocated_sp

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "skill_tables": skill_tables,
    }

    return render(request, 'corptools/character/skills.html', context=context)


@login_required
def clones(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)
    character_ids = characters.values_list('character_id', flat=True)
    jump_clones = JumpClone.objects\
        .filter(character__character__character_id__in=character_ids)\
        .select_related('character__character', 'location_name').prefetch_related('implant_set', 'implant_set__type_name')
    clones = Clone.objects\
        .filter(character__character__character_id__in=character_ids)\
        .select_related('character__character')

    table_data = {}
    for char in characters:
        table_data[char.character_name] = {
            "character": char, "clones": {}, "home": None, "cooldown": 0}

    for j in jump_clones:
        table_data[j.character.character.character_name]["clones"][j.jump_clone_id] = {
            "name": j.name, "location": j.location_name, "implants": j.implant_set.all()}

    # for c in clones:
    #    table_data[c.get('char')]["total_sp"] = c.get('total_sp')

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "table_data": table_data,
    }

    return render(request, 'corptools/character/clones.html', context=context)


@login_required
def roles(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)
    character_ids = characters.values_list('character_id', flat=True)
    roles_data = CharacterRoles.objects\
        .filter(character__character__character_id__in=character_ids)\
        .select_related('character__character')

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "char_data": roles_data,
    }

    return render(request, 'corptools/character/roles.html', context=context)


@login_required
def market(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)
    character_ids = characters.values_list('character_id', flat=True)

    market_data_current = CharacterMarketOrder.objects\
        .filter(character__character__character_id__in=character_ids, state="active")\
        .select_related('character__character', 'type_name', 'location_name')

    market_data_old = CharacterMarketOrder.objects\
        .filter(character__character__character_id__in=character_ids, duration__gt=0)\
        .select_related('character__character', 'type_name', 'location_name')\
        .exclude(state="active")

    total_buy = market_data_current.filter(is_buy_order=True).aggregate(
        total_buy=Sum(F('price')*F('volume_remain'), output_field=FloatField()))
    total_sell = market_data_current.filter(is_buy_order=None).aggregate(
        total_sell=Sum(F('price')*F('volume_remain'), output_field=FloatField()))

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "char_data": market_data_current,
        "old_data": market_data_old,
        "total_sell": total_sell['total_sell'],
        "total_buy": total_buy['total_buy'],
    }

    return render(request, 'corptools/character/market.html', context=context)


@login_required
def notifications(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)
    character_ids = characters.values_list('character_id', flat=True)
    notifications = Notification.objects\
        .filter(character__character__character_id__in=character_ids)\
        .select_related('character__character')

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "table_data": notifications,
    }

    return render(request, 'corptools/character/notifications.html', context=context)


@login_required
def contacts(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)
    character_ids = characters.values_list('character_id', flat=True)
    contacts = CharacterContact.objects\
        .filter(character__character__character_id__in=character_ids)\
        .select_related('character__character', 'contact_name') \
        .prefetch_related('labels')

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "table_data": contacts,
    }

    return render(request, 'corptools/character/contacts.html', context=context)


@login_required
def mail_menu(request, character_id=None):
    # get available models

    main_char, characters, net_worth = get_alts(request, character_id)
    character_ids = characters.values_list('character_id', flat=True)
    notifications = Notification.objects\
        .filter(character__character__character_id__in=character_ids)\
        .select_related('character__character')

    context = {
        "main_char": main_char,
        "alts": characters,
        "net_worth": net_worth,
        "table_data": notifications,
    }

    return render(request, 'corptools/character/mail_menu.html', context=context)



from django.utils.safestring import mark_safe
from django.template.defaulttags import register
from .. import app_settings as app_sett


@register.filter(name='addclass')
def addclass(value, arg):
    return value.as_widget(attrs={'class': arg})


@register.filter(name='subtract')
def subtract(value, arg):
    return value - arg


@register.filter(name='deslug')
def deslug(slugged):
    try:
        return slugged.replace('_', ' ')
    except:
        return slugged


@register.simple_tag()
def app_setting():
    return app_sett


@register.filter(name='level')
def skill_level(active, trained):
    alt_text = "Level %d" % active
    omega_dif = trained-active
    full = '<span class="fas fa-circle"></span> '
    alpha = '<span class="fas fa-circle text-danger"></span> '
    empty = '<span class="far fa-circle"></span> '

    return mark_safe('<div role="text" aria-label="{}">{}{}{}</div>'.format(alt_text, full*active, alpha*omega_dif, empty*(5-(active+omega_dif))))


@register.filter()
def standing_span(standing):
    if standing > 0:
        if standing <= 5:
            return mark_safe('<span class="label label-info">{}</span>'.format(standing))
        else:
            return mark_safe('<span class="label label-primary">{}</span>'.format(standing))
    elif standing < 0:
        if standing >= -5:
            return mark_safe('<spam class="label label-warning">{}</span>'.format(standing))
        else:
            return mark_safe('<span class="label label-danger">{}</span>'.format(standing))
    else:
        return mark_safe('<span class="label label-default">{}</span>'.format(standing))
